package atomicwheel.why;

import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import transportgraph.TransportDataType;
import transportgraph.TransportGraph;
import atomicwheel.Attribute;
import atomicwheel.Edge;
import atomicwheel.Graph;
import atomicwheel.GraphItem;
import atomicwheel.Node;

public abstract class TransportGraphConverter {
	private static final Map<Class<?>, TransportDataType> typeMap = new HashMap<Class<?>, TransportDataType>();
	private static final Map<String, Class<?>> revTypeMap = new HashMap<String, Class<?>>();

	private static void registerTypeMapping(final Class<?> type,
			final TransportDataType transportType) {
		typeMap.put(type, transportType);
		revTypeMap.put(transportType.name(), type);
	}

	static {
		registerTypeMapping(String.class, TransportDataType.stringValue);
		registerTypeMapping(Integer.class, TransportDataType.intValue);
		registerTypeMapping(Double.class, TransportDataType.doubleValue);
		registerTypeMapping(Boolean.class, TransportDataType.booleanValue);
	}

	private TransportGraphConverter() {
	}

	/**
	 * Convert a {@link Graph} to a {@link TransportGraph}. IDs are assigned
	 * automatically.
	 * 
	 * @param graph
	 *            the graph to convert
	 * @return a transport graph
	 */
	public static TransportGraph createTransportGraph(final Graph graph) {
		return createTransportGraph(graph, null, null);
	}

	/**
	 * Convert a {@link Graph} to a {@link TransportGraph}.
	 * 
	 * @param graph
	 *            the graph to convert
	 * @param nodeID
	 *            node IDs, or <code>null</code> to assign automatically
	 * @param edgeID
	 *            edge IDs, or <code>null</code> to assign automatically
	 * @return a transport graph
	 */
	public static TransportGraph createTransportGraph(final Graph graph,
			final Attribute<Node, String> nodeID,
			final Attribute<Edge, String> edgeID) {
		final Node[] nodes = graph.getNodes().toArray(new Node[0]);
		final Edge[] edges = graph.getEdges().toArray(new Edge[0]);
		final Attribute<Node, String> nid = createID(nodeID, nodes);
		final Attribute<Edge, String> eid = createID(edgeID, edges);

		final Attribute<Edge, Boolean> directed = new Attribute<Edge, Boolean>(
				Boolean.class);
		final String[][] transportEdges = new String[edges.length][3];
		for (int i = 0; i < edges.length; i++) {
			final Edge edge = edges[i];
			directed.set(edge, edge.isDirected());
			transportEdges[i][TransportGraph.EDGE_INDEX_SOURCE] = nid.get(edge
					.getSource());
			transportEdges[i][TransportGraph.EDGE_INDEX_TARGET] = nid.get(edge
					.getTarget());
			transportEdges[i][TransportGraph.EDGE_INDEX_ID] = eid.get(edge);
		}

		final TransportGraph tg = new TransportGraph(toArray(nid, nodes),
				transportEdges);
		tg.addEdgeAttribute(new transportgraph.Attribute(
				TransportGraph.DIRECTED_ATTRIBUTE,
				TransportDataType.booleanValue, true, false),
				toArray(directed, edges));

		for (final Entry<String, Attribute<Node, ?>> entry : graph
				.getNodeAttributeManager().getAttributes().entrySet()) {
			final Attribute<Node, ?> attribute = entry.getValue();
			if (attribute == nodeID)
				continue;
			final String name = entry.getKey();
			tg.addNodeAttribute(createTransportAttribute(name, attribute),
					toArray(attribute, nodes));
		}
		for (final Entry<String, Attribute<Edge, ?>> entry : graph
				.getEdgeAttributeManager().getAttributes().entrySet()) {
			final Attribute<Edge, ?> attribute = entry.getValue();
			if (attribute == edgeID)
				continue;
			final String name = entry.getKey();
			tg.addEdgeAttribute(createTransportAttribute(name, attribute),
					toArray(attribute, edges));
		}
		for (final Entry<String, Attribute<Graph, ?>> entry : graph
				.getGraphAttributeManager().getAttributes().entrySet()) {
			final Attribute<Graph, ?> attribute = entry.getValue();
			final String name = entry.getKey();
			tg.addGraphAttribute(createTransportAttribute(name, attribute),
					attribute.get(graph, null));
		}

		return tg;
	}

	private static <GI extends GraphItem> transportgraph.Attribute createTransportAttribute(
			final String name, final Attribute<GI, ?> attribute) {
		final Class<?> type = attribute.getType();
		if (typeMap.containsKey(type))
			return new transportgraph.Attribute(name, typeMap.get(type), false,
					attribute.isList());
		throw new IllegalArgumentException(name + ": " + type);
	}

	private static <GI extends GraphItem> Attribute<GI, String> createID(
			final Attribute<GI, String> inputID, final GI[] items) {
		if (inputID != null)
			return inputID;

		final Attribute<GI, String> id = new Attribute<GI, String>(String.class);
		for (int i = 0; i < items.length; i++)
			id.set(items[i], "" + i);
		return id;
	}

	private static <GI extends GraphItem, T> T[] toArray(
			final Attribute<GI, T> attr, final GI[] items) {
		@SuppressWarnings("unchecked")
		final T[] array = (T[]) Array.newInstance(attr.getType(), items.length);
		for (int i = 0; i < items.length; i++)
			array[i] = attr.get(items[i], null);
		return array;
	}

	/**
	 * Converts a {@link TransportGraph} to a {@link Graph}. IDs are ignored.
	 * 
	 * @param graph
	 *            the transport graph to convert
	 * @return a graph
	 */
	public static Graph createGraph(final TransportGraph graph) {
		return createGraph(graph, null, null);
	}

	/**
	 * Converts a {@link TransportGraph} to a {@link Graph}.
	 * 
	 * @param graph
	 *            the transport graph to convert
	 * @param nodeIDAttribute
	 *            name of the node ID attribute, or <code>null</code> to ignore
	 *            node IDs
	 * @param edgeIDAttribute
	 *            name of the edge ID attribute, or <code>null</code> to ignore
	 *            edge IDs
	 * @return a graph
	 */
	public static Graph createGraph(final TransportGraph graph,
			final String nodeIDAttribute, final String edgeIDAttribute) {
		final Map<String, Node> nodes = new HashMap<String, Node>();
		final String[] nodeIDs = graph.getNodeIDs();
		for (final String nodeID : nodeIDs)
			nodes.put(nodeID, new Node());

		Object[] directed = null;
		for (final transportgraph.Attribute attr : graph
				.getProtectedEdgeAttributes().keySet())
			if (attr.getName().equals(TransportGraph.DIRECTED_ATTRIBUTE)
					&& attr.getType().equals(
							TransportDataType.booleanValue.name())) {
				directed = graph.getProtectedEdgeAttributes().get(attr);
				break;
			}

		final Map<String, Edge> edges = new HashMap<String, Edge>();
		final String[][] transportEdges = graph.getEdges();
		final String[] edgeIDs = new String[transportEdges.length];
		for (int i = 0; i < transportEdges.length; i++) {
			boolean isDirected = false;
			if (directed != null)
				isDirected = (Boolean) directed[i];
			final String[] transportEdge = transportEdges[i];
			final Node source = nodes
					.get(transportEdge[TransportGraph.EDGE_INDEX_SOURCE]);
			final Node target = nodes
					.get(transportEdge[TransportGraph.EDGE_INDEX_TARGET]);
			edgeIDs[i] = transportEdge[TransportGraph.EDGE_INDEX_ID];
			edges.put(edgeIDs[i], new Edge(source, target, isDirected));
		}

		final Graph g = new Graph(nodes.values(), edges.values());
		if (nodeIDAttribute != null) {
			final Attribute<Node, String> nd = new Attribute<Node, String>(
					String.class);
			fromArray(nodes, nodeIDs, nodeIDs, nd);
			g.getNodeAttributeManager().add(nodeIDAttribute, nd);
		}
		if (edgeIDAttribute != null) {
			final Attribute<Edge, String> id = new Attribute<Edge, String>(
					String.class);
			fromArray(edges, edgeIDs, edgeIDs, id);
			g.getEdgeAttributeManager().add(edgeIDAttribute, id);
		}

		for (final Entry<transportgraph.Attribute, Object[]> entry : graph
				.getNodeAttributes().entrySet()) {
			final transportgraph.Attribute transportAttribute = entry.getKey();
			final Attribute<Node, ?> attribute = createAttribute(
					transportAttribute.getType(), transportAttribute.isList());
			fromArray(nodes, nodeIDs, entry.getValue(), attribute);
			g.getNodeAttributeManager().add(transportAttribute.getName(),
					attribute);
		}
		for (final Entry<transportgraph.Attribute, Object[]> entry : graph
				.getEdgeAttributes().entrySet()) {
			final transportgraph.Attribute transportAttribute = entry.getKey();
			final Attribute<Edge, ?> attribute = createAttribute(
					transportAttribute.getType(), transportAttribute.isList());
			fromArray(edges, edgeIDs, entry.getValue(), attribute);
			g.getEdgeAttributeManager().add(transportAttribute.getName(),
					attribute);
		}
		for (final Entry<transportgraph.Attribute, ?> entry : graph
				.getGraphAttributes().entrySet()) {
			final transportgraph.Attribute transportAttribute = entry.getKey();
			final Attribute<Graph, ?> attribute = createAttribute(
					transportAttribute.getType(), transportAttribute.isList());
			fromValue(g, entry.getValue(), attribute);
			g.getGraphAttributeManager().add(transportAttribute.getName(),
					attribute);
		}
		return g;
	}

	@SuppressWarnings("unchecked")
	private static <T> void fromValue(final Graph g, final Object value,
			final Attribute<Graph, T> attribute) {
		attribute.set(g, (T) value);
	}

	@SuppressWarnings("unchecked")
	private static <GI extends GraphItem, T> void fromArray(
			final Map<String, GI> itemMap, final String[] ids,
			final Object[] entry, final Attribute<GI, T> attribute) {
		for (int i = 0; i < ids.length; i++)
			attribute.set(itemMap.get(ids[i]), (T) entry[i]);
	}

	@SuppressWarnings("unchecked")
	private static <GI extends GraphItem, T> Attribute<GI, T> createAttribute(
			final String type, final boolean isList) {
		if (revTypeMap.containsKey(type)) {
			if (!isList)
				return new Attribute<GI, T>((Class<T>) revTypeMap.get(type));
			return new Attribute<GI, T>(List.class, revTypeMap.get(type));
		}
		throw new IllegalArgumentException(type);
	}
}
